package org.pmmik.dao;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;

public abstract class AbstractDao<T, PK extends Serializable> implements Dao<T, PK> {

    private EntityManager entityManager;
    protected Class<T> entityClass;

    /**
     * @param entityManager
     */
    public AbstractDao(EntityManager entityManager) {
        this.entityManager = entityManager;
        ParameterizedType genericsSuperClass = (ParameterizedType) getClass().getGenericSuperclass();
        this.entityClass = (Class<T>) genericsSuperClass.getActualTypeArguments()[0];
    }

    /* (non-Javadoc)
     * @see org.pmmik.itsh.dao.Dao#create(java.lang.Object)
     */
    @Override
    public void create(T t) {
        try {
            this.entityManager.getTransaction().begin();
            this.entityManager.persist(t);
            this.entityManager.getTransaction().commit();
        } catch (Exception e) {
            this.entityManager.getTransaction().rollback();
        }
    }

    /* (non-Javadoc)
     * @see org.pmmik.itsh.dao.Dao#read(java.io.Serializable)
     */
    @Override
    public T read(PK id) {
        return this.entityManager.find(this.entityClass, id);
    }

    /* (non-Javadoc)
     * @see org.pmmik.itsh.dao.Dao#update(java.lang.Object)
     */
    @Override
    public void update(T t) {
        try {
            this.entityManager.getTransaction().begin();
            this.entityManager.merge(t);
            this.entityManager.getTransaction().commit();
        } catch (Exception e) {
            this.entityManager.getTransaction().rollback();
        }
    }

    /* (non-Javadoc)
     * @see org.pmmik.itsh.dao.Dao#delete(java.lang.Object)
     */
    @Override
    public void delete(T t) {
        try {
            this.entityManager.getTransaction().begin();
            T mgd = this.entityManager.merge(t);
            this.entityManager.remove(mgd);
            this.entityManager.getTransaction().commit();
        } catch (Exception e) {
            this.entityManager.getTransaction().rollback();
        }
    }

    /**
     * @return the entityManager
     */
    protected EntityManager getEntityManager() {
        return this.entityManager;
    }

    public List<T> listAll(String tableName) {

        Query query = this.entityManager.createQuery("Select t from " + tableName + " t");
        List<T> dataList = query.getResultList();

        return dataList;
    }
}
