package org.pmmik.main;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.pmmik.dao.PhoneDao;
import org.pmmik.dao.StudentDao;
import org.pmmik.pojo.Phone;
import org.pmmik.pojo.Student;


public class Controller {
	
	private static final String PERSISTENCE_UNIT_NAME = "students"; 
	private static EntityManagerFactory factory;	
	private EntityManager em;
	
	private StudentDao studentDao;
	private PhoneDao phoneDao;

	/**
	 * 
	 */
	public Controller() {
	}

	/**
	 * This method controls the running of program.
	 */
	public void start() {		
	
		try {
			// Initialize of database
			factory = Persistence.createEntityManagerFactory(PERSISTENCE_UNIT_NAME);
			em = factory.createEntityManager();
			
			this.studentDao = new StudentDao(em);
			this.phoneDao = new PhoneDao(em);
			
			System.out.println("\nThe original students: "); 
			printOutAll();

			insertOne();
			System.out.println("\nAfter insert:"); 
			printOutAll();
			
			System.out.println("\nOne selected student by eha-code");
			printOutOneByEha();			
				
			System.out.println("\nThe phones: "); 
			printOutPhones();
			
			System.out.println("\nThe phones of a student:"); 
			printOutStudentPhones();
			
			em.close();			

		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Prints the list of students
	 * 
	 * @throws Exception
	 */
	private void printOutAll() {
		List<Student> studentList = this.studentDao.listAllStudent();
		for (Student student : studentList) {
			System.out.println(student);
		}
	}
	
	/**
	 * Inserts a student
	 * 
	 * @throws Exception 
	 */
	private void insertOne()  {
		Student student = new Student("Hajdú Ernő", "HAERGEP.PTE", 4.3);  
//		 Student student = new Student("Hajdú Ernő", "HOBAVIL.PTE", 4.3); 

		if (this.studentDao.listOne(student.getEha()) == null) {
			this.studentDao.create(student);
		} else {
			System.out.println("\nFailure! Existing eha-code "); 
		}
	}

	/**
	 * Prints one selected student by eha-code.
	 * 
	 * @throws Exception 
	 */
	private void printOutOneByEha()  {
		String eha = "HAERGEP.PTE"; 		
//		String eha = "aaa.PTE"; 
		
		Student student = this.studentDao.listOne(eha);
		System.out.println("\nThe selected student is: " + student); 
	}


	/**
	 * Prints the list of phones
	 */
	private void printOutPhones() {
		List<Phone> phoneList = this.phoneDao.listAllPhone();
		for (Phone phone : phoneList) {
			System.out.println(phone);
		}
	}
	
	/**
	 * Prints the list of a student's phones
	 */
    private void printOutStudentPhones() {
//		String eha = "HAERGEP.PTE"; 	
        String eha = "KOBEINF.PTE";

        List<Phone> phones = this.studentDao.listStudentPhones(eha);

        if (phones != null) {
            if (phones.isEmpty()) {
                System.out.println("\nStudent with eha " + eha
                        + " doesn't have phones:");
            } else {
                System.out
                        .println("\nStudent with eha " + eha + " owns this phone.");
                for (Phone phone : phones) {
                    System.out.println(phone);
                }
            }
        }
    }
}


